package game.objects;

import game.exceptions.Draw;
import game.exceptions.Defeat;
import game.exceptions.Victory;
import game.exceptions.GameResult;

/**
 * Created by IntelliJ IDEA.
 * User: Nuno Rafael
 * Date: 11/Dez/2006
 * Time: 22:56:24
 * To change this template use File | Settings | File Templates.
 */
public class GameBoard
{
	private Column[] columns;
	private String adversaryName;
	private int rowCount;
	private int columnCount;
	private int freeColumns;

	public GameBoard(String adversaryName, int rowCount, int columnCount)
	{
		this.adversaryName = adversaryName;
		this.rowCount = rowCount;
		this.columnCount = columnCount;
		this.freeColumns = columnCount;
		columns = new Column[columnCount];
		for (int columnIndex = 0; columnIndex < columns.length; columnIndex++)
			columns[columnIndex] = new Column(rowCount);
	}

	public String toString()
	{
		return "adversaryName " + adversaryName + ": " + super.toString();	//To change body of overridden methods use File | Settings | File Templates.
	}

	public void drop(boolean me, int columnIndex) throws GameResult
	{
		Column column;
		(column = columns[columnIndex]).drop(me);
		checkSequence(me, column.getDiskCount() - 1, columnIndex);
		if (column.isFull() && (--freeColumns == 0))
			throw new Draw(adversaryName);
	}

	private void checkSequence(boolean me, int rowIndex, int columnIndex) throws GameResult
	{
		int verticalSequence = 0, horizontalSequence = 0, firstDiagonal = 0, secondDiagonal = 0;
		for(int sequence = -3; sequence < 4; sequence++)
		{
			if (rowIndex + sequence >= 0 && rowIndex + sequence < rowCount)
				if (!isEmpty(rowIndex + sequence, columnIndex))
					if (getDisk(rowIndex + sequence, columnIndex) != me)
						verticalSequence = 0;
					else if (++verticalSequence == 4)
						if(me)
							throw new Victory(adversaryName);
						else
							throw new Defeat(adversaryName);

			if (columnIndex + sequence >= 0 && columnIndex + sequence < columnCount)
				if (!isEmpty(rowIndex, columnIndex + sequence))
					if (getDisk(rowIndex, columnIndex + sequence) != me)
						horizontalSequence = 0;
					else if (++horizontalSequence == 4)
						if(me)
							throw new Victory(adversaryName);
						else
							throw new Defeat(adversaryName);

			if (rowIndex + sequence >= 0 && rowIndex + sequence < rowCount && columnIndex + sequence >= 0 && columnIndex + sequence < columnCount)
				if (!isEmpty(rowIndex + sequence, columnIndex + sequence))
					if (getDisk(rowIndex + sequence, columnIndex + sequence) != me)
						firstDiagonal = 0;
					else if (++firstDiagonal == 4)
						if(me)
							throw new Victory(adversaryName);
						else
							throw new Defeat(adversaryName);

			if (rowIndex - sequence >= 0 && rowIndex - sequence < rowCount && columnIndex + sequence >= 0 && columnIndex + sequence < columnCount)
				if (!isEmpty(rowIndex - sequence, columnIndex + sequence))
					if (getDisk(rowIndex - sequence, columnIndex + sequence) != me)
						secondDiagonal = 0;
					else if (++secondDiagonal== 4)
						if(me)
							throw new Victory(adversaryName);
						else
							throw new Defeat(adversaryName);
		}
	}

	public boolean getDisk(int rowIndex, int columnIndex)
	{
		return columns[columnIndex].getDisk(rowIndex);
	}

	public boolean isEmpty(int rowIndex, int columnIndex)
	{
		return columns[columnIndex].isEmpty(rowIndex);
	}

	public boolean isFull(int columnIndex)
	{
		return columns[columnIndex].isFull();
	}
}
