package game;

import game.exceptions.*;
import game.objects.LoggedPlayer;
import game.objects.Message;
import game.objects.Rank;
import game.objects.ResultType;
import game.rmi.RmiPlayerInterface;
import game.rmi.RmiServerInterface;

import javax.swing.*;
import java.io.*;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.ServerException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Random;
import java.util.Vector;
import java.util.Hashtable;

/**
 * Created by IntelliJ IDEA.
 * User: Nuno Rafael
 * Date: 9/Dez/2006
 * Time: 22:43:38
 * To change this template use File | Settings | File Templates.
 */
public class GameServer extends UnicastRemoteObject implements RmiServerInterface
{
	private MyTable<LoggedPlayer> jTableLoggedPlayers;
	private MyTable<Rank> jTableRanking;

	public GameServer() throws RemoteException
	{
		this(false);
	}

	public GameServer(boolean visible) throws RemoteException
	{
		super();
		init(visible);
		try
		{
			System.getProperties().put("java.security.policy", "policy.all");
			System.setSecurityManager(new RMISecurityManager());
			Naming.rebind("Servidor4", this);
			System.out.println("Server listening");
		}
		catch (RemoteException e)
		{
			// nothing to do
			e.printStackTrace();
		}
		catch (MalformedURLException e)
		{
			// nothing to do...
			e.printStackTrace();
		}
	}

	private void init(boolean visible) throws ServerException
	{
		JTabbedPane
				jTabbedPane;

		JFrame jFrame = new JFrame();
		jFrame.getContentPane().add(jTabbedPane = new JTabbedPane());
		jTabbedPane.add("Logged Players", jTableLoggedPlayers = new MyTable<LoggedPlayer>("data/logged.txt", LoggedPlayer.columns));
		jTabbedPane.add("Ranking", jTableRanking = new MyTable<Rank>("data/ranking.txt",Rank.columns));

		try
		{
			BufferedReader bufferedReader = new BufferedReader(new FileReader("data/ranking.txt"));
			String line;
			try
			{
				while((line = bufferedReader.readLine()) != null)
					save(new Rank(line));
			}
			catch (IOException e)
			{
				throw new ServerException("Server Exception", e);
			}
		}
		catch (FileNotFoundException e)
		{
			//nothing --> it's all ok
		}
		jFrame.setVisible(visible);
		jFrame.pack();
	}

	static
	{
		File folder = new File("data");
		if (!folder.exists())
			folder.mkdir();
	}

	public void register(RmiPlayerInterface rmiPlayerInterface, String playerName, String password, String channel) throws RegisterException, ServerException
	{
		BufferedReader bufferedReader;
		try
		{
			bufferedReader = new BufferedReader(new FileReader("data/players.txt"));
			String line;
			while((line = bufferedReader.readLine()) != null)
				if (line.split(" ")[0].equalsIgnoreCase(playerName))
				{
					bufferedReader.close();
					throw new RegisterException(playerName);
				}

			bufferedReader.close();
		}
		catch (FileNotFoundException e)
		{
			//nothing --> it's all ok
		}
		catch (IOException e)
		{
			throw new ServerException("Server Exception", e);
		}
		PrintWriter printWriter;
		try
		{
			printWriter = new PrintWriter(new FileWriter("data/players.txt", true));
		}
		catch (IOException e)
		{
			throw new ServerException("Server Exception", e);
		}
		printWriter.println(playerName + " " + password);
		printWriter.close();

		add(playerName, rmiPlayerInterface, channel);
	}

	public void login(RmiPlayerInterface rmiPlayerInterface, String playerName, String password, String channel) throws LoginException, ServerException
	{
		if (isPlayerLogged(playerName))
			throw new PlayerAlreadyLoggedException(playerName);
		File folder = new File("data");
		if (!folder.exists())
			throw new PlayerNotRecognizedException(playerName);
		BufferedReader bufferedReader;
		try
		{
			bufferedReader = new BufferedReader(new FileReader("data/players.txt"));
		}
		catch (FileNotFoundException e)
		{
			throw new PlayerNotRecognizedException(playerName);
		}
		String line;
		String[] splits;
		try
		{
			while((line = bufferedReader.readLine()) != null)
				if ((splits = line.split(" "))[0].equalsIgnoreCase(playerName))
				{
					bufferedReader.close();
					if (splits[1].equals(password))
					{
						add(playerName, rmiPlayerInterface, channel);
						return;
					}
					throw new PlayerNotRecognizedException(playerName);
				}
		}
		catch (IOException e)
		{
			throw new ServerException("Server Exception", e);
		}

		throw new PlayerNotRecognizedException(playerName);
	}

	public void logout(String playerName) throws ServerException
	{
		jTableLoggedPlayers.remove(playerName);
		update();
	}

	public void gameChat(Message message) throws AdversaryLoggedOffException, ServerException
	{
		try
		{
			getLoggedPayerInterface(message.getReceiver()).beGameChatted(message);
		}
		catch (RemoteException e)
		{
			update();
		}
	}

	public void mainChat(Message message) throws ServerException
	{
		for(LoggedPlayer loggedPlayer : getLoggedPlayers())
			try
			{
				loggedPlayer.rmiPlayerInterface.beMainChatted(message);
			}
			catch (RemoteException e)
			{
				update();
			}
	}

	public void invite(String inviter, String invitee) throws AdversaryLoggedOffException, ServerException
	{
		try
		{
			getLoggedPayerInterface(invitee).beInvited(inviter);
		}
		catch (RemoteException e)
		{
			update();
		}
	}

	public void answerInvitation(String invitee, String inviter, boolean accept) throws AdversaryLoggedOffException, ServerException
	{
		try
		{
			getLoggedPayerInterface(inviter).seeInvitationAnswered(invitee, accept);
			if(new Random(System.currentTimeMillis()).nextBoolean())
			{
				getLoggedPayerInterface(inviter).startGame(invitee, true);
				getLoggedPayerInterface(invitee).startGame(inviter, false);
			}
			else
			{
				getLoggedPayerInterface(inviter).startGame(invitee, false);
				getLoggedPayerInterface(invitee).startGame(inviter, true);
			}

		}
		catch (RemoteException e)
		{
			update();
		}
	}

	public void drop(String playerName, String adversaryName, int columnIndex) throws ServerException, AdversaryLoggedOffException
	{
		try
		{
			getLoggedPayerInterface(adversaryName).adversaryDrop(playerName, columnIndex);
		}
		catch (RemoteException e)
		{
			update();
		}
	}

	public void endGame(String name, GameResult gameResult) throws RemoteException
	{
		String adversaryName = gameResult.getAdversaryName();
		int first, second;
		try
		{
			switch(gameResult.getResultType())
			{
				case DEFEAT:
					getLoggedPayerInterface(adversaryName).endGame(new GameResult(name, ResultType.VICTORY));
					break;
				case VICTORY:
					getLoggedPayerInterface(adversaryName).endGame(new GameResult(name, ResultType.DEFEAT));
					break;
				default:
					getLoggedPayerInterface(adversaryName).endGame(new GameResult(name, ResultType.DRAW));
			}
		}
		catch (AdversaryLoggedOffException e)
		{
			//
		}
		if (gameResult.getResultType() == ResultType.DEFEAT)
		{
			if ((first = getRank(name)) < (second = getRank(adversaryName)))
			{
				save(name, first - 1);
				save(adversaryName, second + 2);
			}
			else if (first > second)
			{
				save(name, first - 2);
				save(adversaryName, second + 4);
			}
			else
			{
				save(name, first - 1);
				save(adversaryName, second + 3);
			}
		}
		else
		{
			if ((first = getRank(name)) < (second = getRank(adversaryName)))
				save(name, first + 2);
			else if (first > second)
				save(adversaryName, second + 2);
			else
			{
				save(name, first + 1);
				save(adversaryName, second + 1);
			}
		}
	}

	private int getRank(String name)
	{
		return jTableRanking.get(name).getPoints();
	}

	private void save(String name, int points) throws ServerException
	{
		save(new Rank(name, points));
	}

	private void save(Rank rank) throws ServerException
	{
		jTableRanking.save(rank);
		update();
	}

	public void add(String playerName, RmiPlayerInterface rmiPlayerInterface, String channel) throws ServerException
	{
		save(new LoggedPlayer(playerName, rmiPlayerInterface, channel));
	}

	private void save(LoggedPlayer loggedPlayer) throws ServerException
	{
		jTableLoggedPlayers.save(loggedPlayer);
		update();
	}

	public LoggedPlayer[] getLoggedPlayers()
	{
		Vector<LoggedPlayer> rows;
		LoggedPlayer[] loggedPlayers = new LoggedPlayer[(rows = jTableLoggedPlayers.getRows()).size()];
		for(int index = 0; index < rows.size(); index++)
			loggedPlayers[index] = rows.get(index);
		return loggedPlayers;
	}

	public void remove(LoggedPlayer loggedPlayer) throws ServerException
	{
		jTableLoggedPlayers.remove(loggedPlayer.getPlayerName());
	}

	public Hashtable<String, Rank> getRanking()
	{
		return jTableRanking.getHashtable();
	}

	public void update() throws ServerException
	{
		for(LoggedPlayer loggedPlayer : getLoggedPlayers())
			try
			{
				loggedPlayer.rmiPlayerInterface.beUpdated(getLoggedPlayers(), getRanking());
			}
			catch (RemoteException e)
			{
				remove(loggedPlayer);
				update();
				break;
			}
	}

	public boolean isPlayerLogged(String playerName)
	{
		for(LoggedPlayer loggedPlayer : getLoggedPlayers())
			if (loggedPlayer.getPlayerName().equalsIgnoreCase(playerName))
				return true;
		return false;
	}

	public RmiPlayerInterface getLoggedPayerInterface(String playerName) throws AdversaryLoggedOffException, ServerException
	{
		if (isPlayerLogged(playerName))
			return getLoggedPlayer(playerName).rmiPlayerInterface;
		else
		{
			update();
			throw new AdversaryLoggedOffException(playerName);
		}
	}

	private LoggedPlayer getLoggedPlayer(String playerName)
	{
		return jTableLoggedPlayers.get(playerName);
	}

	public static void main(String[] args) throws RemoteException
	{
		new GameServer(args.length != 0 && args[0].equals("visible"));
	}
}
