package game;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import javax.swing.*;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.rmi.ServerException;
import java.util.Hashtable;
import java.util.Vector;

/**
 * Created by IntelliJ IDEA.
 * User: Nuno Rafael
 * Date: 8/Jan/2007
 * Time: 16:44:17
 * To change this template use File | Settings | File Templates.
 */
public class MyTable<Row extends MyElement> extends JScrollPane
{
	private final Vector<Row> rows = new Vector<Row>();
	private Hashtable<String, Row> hashtable = new Hashtable<String, Row>();
	private String fileName;

	public MyTable(String fileName, final Vector<MyColumnType> columns)
	{
		this.fileName = fileName;
		setViewportView(new JTable(new TableModel()
		{
			public int getRowCount()
			{
				return rows.size();
			}

			public int getColumnCount()
			{
				return 2;
			}

			public String getColumnName(int columnIndex)
			{
				return columns.get(columnIndex).getColumnName();
			}

			public Class<?> getColumnClass(int columnIndex)
			{
				return columns.get(columnIndex).getColumnClass();
			}

			public boolean isCellEditable(int rowIndex, int columnIndex)
			{
				return false;
			}

			public Object getValueAt(int rowIndex, int columnIndex)
			{
				if (rowIndex > rows.size())
					return null;
				return rows.get(rowIndex).getValueAt(columnIndex);
			}

			public void setValueAt(Object aValue, int rowIndex, int columnIndex)
			{
				throw new NotImplementedException();
			}

			public void addTableModelListener(TableModelListener l)
			{
			}

			public void removeTableModelListener(TableModelListener l)
			{
			}
		}));
	}

	public void load(Row row)
	{
		Row old;
		if (rows.contains(old = hashtable.get(row.getUserName())))
			rows.remove(old);

		hashtable.put(row.getUserName(), row);
		rows.add(row);
	}

	public void save(Row row) throws ServerException
	{
		load(row);
		save();
	}

	private void save() throws ServerException
	{
		PrintWriter printWriter;
		try
		{
			printWriter = new PrintWriter(new FileWriter(fileName));
		}
		catch (IOException e)
		{
			throw new ServerException("Server Exception", e);
		}
		for(Row row : rows)
			printWriter.println(row);
		printWriter.close();
	}

	public void remove(String key) throws ServerException
	{
		if (!hashtable.containsKey(key))
			return;
		Row row = hashtable.remove(key);
		rows.remove(row);
		save();
	}

	public Row get(String key)
	{
		return hashtable.get(key);
	}

	public Vector<Row> getRows()
	{
		return rows;
	}

	public Hashtable<String, Row> getHashtable()
	{
		return hashtable;
	}
}
