package game;

import game.exceptions.*;
import game.objects.*;
import game.rmi.RmiPlayerInterface;
import gui.*;

import javax.swing.*;
import java.io.IOException;
import java.net.DatagramPacket;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.rmi.NotBoundException;
import java.rmi.RMISecurityManager;
import java.util.Hashtable;
import java.util.Vector;


/**
 *
 *
 * @author esta classe  apenas para testar a interface. Pode ser alterada
 */
public class Game extends UnicastRemoteObject implements LoginInterface, MainInterface, GameWindowInterface, RmiPlayerInterface
{
	private LoginWindow loginWindow;
	private MainWindow mainWindow;
	private Hashtable<String, GameWindow> gameWindows = new Hashtable<String, GameWindow>();
	private String playerName;
	private AbstractPlayer player;

	private Hashtable<String, AdversaryInfo> adversariesInfo;

	static
	{
		System.setProperty("java.security.policy", "policy.all");
		System.setSecurityManager(new RMISecurityManager());
	}

	public Game(ConnectionType type) throws IOException
	{
		super();
		switch(type)
		{
/*			case TCP: //todo
				player = new TCPPlayer(this);
				break;*/
			default:
				player = new RmiPlayer(this);
		}

		loginWindow = new LoginWindow(this);
		adversariesInfo = new Hashtable<String, AdversaryInfo>();
	}

	public void beMainChatted(Message message) throws RemoteException
	{
		mainWindow.mainChat(message.getSender() + "> " + message.getContent() + "\n");
	}

	public void seeInvitationAnswered(String invited, boolean accept) throws RemoteException
	{
		adversariesInfo.get(invited).status = accept ? AdversaryStatus.ACCEPTED_ME : AdversaryStatus.REFUSED_ME;
	}

	public void beGameChatted(Message message) throws RemoteException
	{
		adversariesInfo.get(message.getSender()).getGameChat().add(message);
	}

	public Vector<Message> getGameChat(String adversaryName) throws AdversaryLoggedOffException
	{
		return adversariesInfo.get(adversaryName).getGameChat();
	}

	public void loginConfirmed(String userName)
	{
		this.playerName = userName;
		loginWindow.setEnabled(false);
		loginWindow.setVisible(false);
		mainWindow = new MainWindow(userName, this);
	}

	public void beAccepted(final String userName)
	{
		if (!gameWindows.containsKey(userName))
			return;
		SwingUtilities.invokeLater(new Runnable() {
			 public void run() {
				 mainWindow.showMessageBox("O jogador " + userName + " aceitou o seu convite", "Resposta a Convite", JOptionPane.INFORMATION_MESSAGE);
			 }
		});
	}

	public void beRefused(final String userName)
	{
		if (!gameWindows.containsKey(userName))
			return;
		SwingUtilities.invokeLater(new Runnable() {
			 public void run() {
				 mainWindow.showMessageBox("O jogador " + userName + " recusou o seu convite", "Resposta a Convite", JOptionPane.INFORMATION_MESSAGE);
			 }
		});
	}

	public void beChatted(final String userName, final String message)
	{
		if (!gameWindows.containsKey(userName))
			return;
		 SwingUtilities.invokeLater(new Runnable() {
			 public void run() {
				 gameWindows.get(userName).addTextToBuffer(message);
			 }
		 });
	}

	public String getPlayerName()
	{
		return playerName;
	}

	public void newGame(boolean first, String otherUserName)
	{
		gameWindows.put(otherUserName, new GameWindow(first, playerName, otherUserName, this));
	}

	private synchronized void verificaSair()
	{
		if (loginWindow != null && loginWindow.isVisible())
			return;
		if (mainWindow != null && mainWindow.isVisible())
			return;
		if (gameWindows.size() == 0)
			System.exit(0);
	}

	public static void main(String[] args) throws IOException
	{
		if (args.length > 0 && args[0].equals("RMI"))
			new Game(ConnectionType.RMI);
		else
			new Game(ConnectionType.TCP);
	}

	public synchronized void gameWindowSair(GameWindow gameWindow) throws IOException
	{
		if (!gameWindow.checkGameOver())
			player.endGame(new GameResult(gameWindow.getOtherUserName(), ResultType.DEFEAT));
		gameWindows.remove(gameWindow.getOtherUserName());

		verificaSair();
	}

	public synchronized void gameWindowFimDoJogo(GameWindow gui, String winner, String looser)
	{
		if (winner.equalsIgnoreCase(playerName))
			try
			{
				player.endGame(new GameResult(looser, ResultType.VICTORY));
			}
			catch (RemoteException e)
			{
				//todo
			}
	}

	public synchronized void gameWindowDesistir(GameWindow gui) throws IOException
	{
		gameWindowSair(gui);
	}

	public synchronized void gameWindowNovoJogo(GameWindow gui)
	{
		System.out.println("Novo jogo");
		gui.resetBoard();
	}

	public synchronized void gameWindowEnviaMensagem(GameWindow gui, String text) throws IOException
	{
		try
		{
			player.gameChat(new Message(gui.getOtherUserName(), text));
		}
		catch (AdversaryLoggedOffException e)
		{
			//todo
		}
	}

	public void invite(String invitee) throws RemoteException, AdversaryLoggedOffException
	{
		player.invite(invitee);
		adversariesInfo.get(invitee).status = AdversaryStatus.INVITED_BY_ME;
	}

	public synchronized void gameWindowJogarPeca(GameWindow gui, int column) throws IOException
	{
		try
		{
			player.drop(gui.getOtherUserName(),column);
		}
		catch (AdversaryLoggedOffException e)
		{
			//todo
		}
	}

	public synchronized void loginWindowBotaoLigar(String userName, String passWord) throws IOException
	{
		player.setName(userName);
		player.setPassword(passWord);
		player.setServer("localhost");
		try
		{
			player.login();
		}
		catch (NotBoundException e)
		{
			//todo
		}
		catch (LoginException e)
		{
			//todo
		}
	}

	public synchronized void loginWindowBotaoRegistar(String userName, String passWord) throws IOException
	{
		try
		{
			player.setServer("localhost");
			player.setPlayerName(userName);
			player.setPassword(passWord);
			player.register();
		}
		catch (NotBoundException e)
		{
			//todo
		}
		catch (RegisterException e)
		{
			//todo
		}
	}

	// todo est tudo doido?
	public synchronized void loginWindowSair()
	{
		System.out.println("fechou a janela loginWindow");

		verificaSair();
	}

	public synchronized void mainWindowDuploCliqueCliente(String invitee) throws IOException
	{
		if (mainWindow.showConfirmBox("Est prestes a convidar o jogador " + invitee + " para uma jogatina.\nConfirma?", "Convidar jogador"))
			try
			{
				player.invite(invitee);
			}
			catch (AdversaryLoggedOffException e)
			{
				//todo
			}
	}

	public synchronized void mainWindowEnviaMensagem(String message)
	{
		try
		{
			player.mainChat(new Message(playerName, message));
		}
		catch (RemoteException e)
		{
			//todo
		}
	}

	public synchronized void mainWindowMenuLigar()
	{
		System.out.println("ficheiro>ligar");
	}

	public synchronized void mainWindowMenuDesligar()
	{
		System.out.println("ficheiro>desligar");

	}

	public synchronized void mainWindowMenuSair()
	{
		System.out.println("ficheiro>sair");

	}

	public synchronized String mainWindowCredits()
	{
		System.out.println("Ajuda>acerca");
		return "    Hugo Gomes\n    Nuno Rafael Figueiredo\n    Tiago Mota";
	}

	public synchronized void mainWindowSair()
	{
		System.out.println("fechou a janela principal");

		//verificaSair();
		System.exit(0);
	}

	public void start(final String otherPlayer, final boolean playerOne)
	{
		final GameWindowInterface gameWindowInterface = this;

		 SwingUtilities.invokeLater(new Runnable() {
			 public void run() {
				 gameWindows.put(otherPlayer, new GameWindow(playerOne, playerName, otherPlayer, gameWindowInterface));
			 }
		 });
	}

	public void playerQuitted(final String quitter)
	{
		 SwingUtilities.invokeLater(new Runnable() {
			 public void run() {
				 gameWindows.get(quitter).showMessageBox("O " + quitter + " desistiu !", "Fim do Jogo", JOptionPane.INFORMATION_MESSAGE);
			 }
		 });
	}

	public void playerWon(final String winner)
	{
		 SwingUtilities.invokeLater(new Runnable() {
			 public void run() {
				 gameWindows.get(winner).showMessageBox("O " + winner + " desistiu !", "Fim do Jogo", JOptionPane.INFORMATION_MESSAGE);
			 }
		 });
	}

	public void playerLost(final String looser)
	{
		 SwingUtilities.invokeLater(new Runnable() {
			 public void run() {
				 gameWindows.get(looser).showMessageBox("O " + looser + " Venceu !", "Fim do Jogo", JOptionPane.INFORMATION_MESSAGE);
			 }
		 });
	}

	public void playerPlayed(final String player, final int column)
	{
		 SwingUtilities.invokeLater(new Runnable() {
			 public void run() {
				 gameWindows.get(player).colocaPeca(column);
			 }
		 });
	}

	public void draw(String userName)
	{
		final String o = userName;

		 SwingUtilities.invokeLater(new Runnable() {
			 public void run() {
				 gameWindows.get(o).showMessageBox("Empatou!", "Fim do Jogo", JOptionPane.INFORMATION_MESSAGE);
			 }
		 });
	}

	public void report(final Exception e)
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				if (loginWindow != null)
					loginWindow.report(e);
				else
					mainWindow.report(e);
			}
		});
	}

	public void adversaryDrop(String adversaryName, int columnIndex) throws RemoteException
	{
		try
		{
			adversariesInfo.get(adversaryName).drop(false, columnIndex);
		}
		catch (Victory victory)
		{
			// todo victory: celebrate
		}
		catch (GameResult gameResult)
		{
			player.endGame(gameResult);
		}
	}

	public AdversaryInfo getAdversaryInfo(String adversaryName)
	{
		return adversariesInfo.get(adversaryName);
	}

	public Hashtable<String, AdversaryInfo> getAdversariesInfo()
	{
		return adversariesInfo;
	}

	public void startGame(String adversaryName, boolean myTurn) throws RemoteException
	{
		adversariesInfo.get(adversaryName).startGame(myTurn);
	}

	public void endGame(GameResult gameResult) throws RemoteException
	{
		adversariesInfo.get(gameResult.getAdversaryName()).setGameResult(gameResult);
	}

	public void beInvited(final String inviter)
	{
		adversariesInfo.get(inviter).status = AdversaryStatus.INVITING_ME;
		if (!gameWindows.containsKey(inviter))
			return;

		 SwingUtilities.invokeLater(new Runnable() {
			 public void run() {
				 try
				 {
					 player.answerInvitation(inviter, mainWindow.showConfirmBox("O jogador " + inviter + " convida-o - para jogar.\nAceita?", "Convite para jogar"));
				 }
				 catch (RemoteException e)
				 {
					 //todo
				 }
				 catch (AdversaryLoggedOffException e)
				 {
					 //todo
				 }
			 }
		 });
	}

	public void log(String s)
	{
		//todo
	}

	public void sending(DatagramPacket datagramPacket)
	{
		log("Sending: " + datagramPacket.getData().toString().substring(0, datagramPacket.getLength()));
	}

	public void logReceive(DatagramPacket datagramPacket)
	{
		log("logReceive: " + datagramPacket.getData().toString().substring(0, datagramPacket.getLength()));
	}

	public void report(final Exception e, final String player)
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				gameWindows.get(player).report(e);
			}
		});
	}

	public void drop(String adversaryName, int columnIndex) throws RemoteException, AdversaryLoggedOffException
	{
		player.drop(adversaryName, columnIndex);
		try
		{
			adversariesInfo.get(adversaryName).drop(true, columnIndex);
		}
		catch (GameResult gameResult)
		{
			//todo
		}
	}

	public void beUpdated(LoggedPlayer[] loggedPlayers, Hashtable<String, Rank> ranking)
	{
		if (mainWindow == null)
			return;

		String adversaryName;
		for(LoggedPlayer loggedPlayer : loggedPlayers)
			if (!adversariesInfo.containsKey(adversaryName = loggedPlayer.getUserName()))
				adversariesInfo.put(adversaryName, new AdversaryInfo(loggedPlayer));

		adversariesInfo.remove(playerName);
		mainWindow.setRanking(ranking);
		for(GameWindow gameWindow : gameWindows.values())
			gameWindow.update(ranking.get(playerName).getPoints(), ranking.get(gameWindow.getOtherUserName()).getPoints());
	}

	public void gameChat(Message message) throws RemoteException, AdversaryLoggedOffException
	{
		player.gameChat(message);
		adversariesInfo.get(message.getReceiver()).getGameChat().add(message);
	}
}