package game;

import game.exceptions.AdversaryLoggedOffException;
import game.exceptions.GameResult;
import game.exceptions.LoginException;
import game.exceptions.RegisterException;
import game.objects.Message;
import game.rmi.RmiPlayerInterface;
import game.rmi.RmiServerInterface;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

public class RmiPlayer extends UnicastRemoteObject implements AbstractPlayer
{
	private String
		server,
		name,
		password;

	private RmiServerInterface rmiServerInterface;
	private RmiPlayerInterface rmiPlayerInterface;

	public String toString()
	{
		return name + ": " + super.toString();
	}

	public RmiPlayer(RmiPlayerInterface rmiPlayerInterface) throws RemoteException
	{
		super();
		this.rmiPlayerInterface = rmiPlayerInterface;
	}

	public void setServer(String server)
	{
		this.server = server;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	public void setPassword(String password)
	{
		this.password = password;
	}

	public void login() throws NotBoundException, LoginException, RemoteException
	{
		getRmiServerInterface().login(rmiPlayerInterface, name, password, "jsp");
	}

	public void register() throws NotBoundException, RegisterException, RemoteException
	{
		getRmiServerInterface().register(rmiPlayerInterface, name, password, "jsp");
	}

	public String getName()
	{
		return name;
	}

	public void logout() throws RemoteException
	{
		getRmiServerInterface().logout(name);
	}

	public void gameChat(Message message) throws RemoteException, AdversaryLoggedOffException
	{
		getRmiServerInterface().gameChat(message);
	}

	public void mainChat(Message message) throws RemoteException
	{
		getRmiServerInterface().mainChat(message);
	}

	public RmiServerInterface getRmiServerInterface()
	{
		if (rmiServerInterface == null)
			try
			{
				rmiServerInterface = (RmiServerInterface) Naming.lookup("rmi://" + server + "/Servidor4");
			}
			catch (NotBoundException e)
			{
				//nothing: no d, no d
			}
			catch (MalformedURLException e)
			{
				//nothing: no d, no d
			}
			catch (RemoteException e)
			{
				//nothing: no d, no d
			}

		return rmiServerInterface;
	}

	public void invite(String invitee) throws RemoteException, AdversaryLoggedOffException
	{
		getRmiServerInterface().invite(name, invitee);
	}

	public void answerInvitation(String inviter, boolean accept) throws RemoteException, AdversaryLoggedOffException
	{
		getRmiServerInterface().answerInvitation(name, inviter, accept);
	}

	public void drop(String adversaryName, int columnIndex) throws RemoteException, AdversaryLoggedOffException
	{
		getRmiServerInterface().drop(name, adversaryName, columnIndex);
	}

	public void setPlayerName(String playerName)
	{
		setName(playerName);
	}

	public void endGame(GameResult gameResult) throws RemoteException
	{
		rmiServerInterface.endGame(name, gameResult);
	}
}
