package gui;


import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.IOException;


/**
 * @author Jorge Farinha
 * @author Tiago Santos
 * @version 1.2
 * @date 24-10-2006
 */
public class GameListener implements MouseListener, WindowListener, ActionListener, KeyListener, MouseMotionListener
{
	private int actionCode = 0;
	private Object sender = null;
	private GameWindowInterface gameWindowInterface;
	private MainInterface mainInterface;
	private LoginInterface loginInterface;
	public static final int ERRO = JOptionPane.ERROR_MESSAGE;

	/**
	 * @param actionCode cdigo da aco a executar
	 */
	public GameListener(int actionCode)
	{
		this.actionCode = actionCode;
	}

	/**
	 * @param actionCode cdigo da aco a executar
	 * @param sender	 objecto em que  gerada a aco
	 */
	public GameListener(int actionCode, Object sender)
	{
		this.actionCode = actionCode;
		this.sender = sender;
	}

	/**
	 * @param actionCode cdigo da aco a executar
	 * @param sender	 objecto em que  gerada a aco
	 * @param gwm		o objecto com os mtodos da interface da janela do jogo
	 */
	public GameListener(int actionCode, Object sender, GameWindowInterface gwm)
	{
		this.actionCode = actionCode;
		this.sender = sender;
		this.gameWindowInterface = gwm;
	}

	/**
	 * @param actionCode cdigo da aco a executar
	 * @param sender	 objecto em que  gerada a aco
	 * @param lwm		o objecto com os mtodos da interface da janela do login
	 */
	public GameListener(int actionCode, Object sender, LoginInterface lwm)
	{
		this.actionCode = actionCode;
		this.sender = sender;
		this.loginInterface = lwm;
	}

	/**
	 * @param actionCode cdigo da aco a executar
	 * @param sender	 objecto em que  gerada a aco
	 * @param mwm		o objecto com os mtodos da interface da janela principal
	 */
	public GameListener(int actionCode, Object sender, MainInterface mwm)
	{
		this.actionCode = actionCode;
		this.sender = sender;
		this.mainInterface = mwm;
	}

	/**
	 * @param actionCode cdigo da aco a executar
	 * @param sender	 objecto em que  gerada a aco
	 * @param gwm		o objecto com os mtodos da interface da janela do jogo
	 * @param mwm		o objecto com os mtodos da interface da janela principal
	 * @param lwm		o objecto com os mtodos da interface da janela do login
	 */
	public GameListener(int actionCode, Object sender, GameWindowInterface gwm, MainInterface mwm, LoginInterface lwm)
	{
		this.actionCode = actionCode;
		this.sender = sender;
		this.gameWindowInterface = gwm;
		this.mainInterface = mwm;
		this.loginInterface = lwm;
	}
	/*
		 public void connect(GameWindowInterface gwm) {
			 gameWindowInterface = gwm;
		 }
		 */

	public void mouseClicked(MouseEvent e)
	{
		if (actionCode == 32)
		{
			GameWindow gameWindow;
			if ((gameWindow = (GameWindow) sender).isMyTurn())
			{
				int col = calcCol(e.getX());
				if (col != -1)
				{
					Point p = gameWindow.colocaPeca(col);
					if (!p.equals(new Point(-1, -1)))
					{
						try
						{
							gameWindowInterface.gameWindowJogarPeca((GameWindow) sender, col);
							gameWindow.ganhou(p);
						}
						catch (IOException e1)
						{
							gameWindow.showMessageBox(e1.getMessage(), e1.getClass().toString(), ERRO);
						}
					}
					gameWindow.checkGameOver();
				}
			}
		}
		else if (actionCode == 23)
		{
			if (e.getClickCount() == 2 && e.getButton() == MouseEvent.BUTTON1)
			{
				String nome = ((MainWindow) sender).getSelectedPlayer();
				try
				{
					mainInterface.mainWindowDuploCliqueCliente(nome);
				}
				catch (IOException e1)
				{
					((MainWindow) sender).showMessageBox(e1.getMessage(), e1.getClass().toString(), ERRO);
				}
			}
		}
	}

	/**
	 * @param x a coordenada do eixo do X do tabuleiro de jogo para a qual se
	 *          quer calcular a coluna do jogo.
	 * @return um inteiro entre 0 e 6 se a coordenada de X for vlida ou -1 se
	 *         for invlida.
	 */
	private int calcCol(int x)
	{
		int offset = 15;
		int col = -1;
		if ((x - offset >= 0) && (x - offset < 560))
		{
			col = (x - offset) / 80;
		}
		return col;
	}


	public void mouseEntered(MouseEvent e)
	{
	}

	public void mouseExited(MouseEvent e)
	{
	}

	public void mousePressed(MouseEvent e)
	{
	}

	public void mouseReleased(MouseEvent e)
	{
	}

	public void windowActivated(WindowEvent e)
	{
	}

	public void windowClosed(WindowEvent e)
	{
	}

	public void windowClosing(WindowEvent e)
	{
		if (this.actionCode == 16)
		{
			loginInterface.loginWindowSair();
		}
		else if (this.actionCode == 31)
		{
			//fechar a janela do jogo
			try
			{
				gameWindowInterface.gameWindowSair((GameWindow) sender);
			}
			catch (IOException e1)
			{
				((GameWindow) sender).showMessageBox(e1.getMessage(), e1.getClass().toString(), ERRO);
			}
		}
		else if (actionCode == 20)
		{
			//fechar a janela principal
			mainInterface.mainWindowSair();
		}
	}

	//private GameWindowInterface gameWindowInterface;
	public void windowDeactivated(WindowEvent e)
	{
	}

	public void windowDeiconified(WindowEvent e)
	{
	}

	public void windowIconified(WindowEvent e)
	{
	}

	public void windowOpened(WindowEvent e)
	{
	}

	public void actionPerformed(ActionEvent e)
	{
		LoginWindow loginWindow;
		switch (actionCode)
		{
			case 14:
				loginWindow = ((LoginWindow) sender);
				try
				{
					if (loginWindow.getLoginName().length() > 0)
						loginInterface.loginWindowBotaoLigar(loginWindow.getLoginName(), loginWindow.getLoginPassword());
					else
						loginWindow.showMessageDialog("Escreva qualquer coisa!!!", "Login invlido", ERRO);
				}
				catch (IOException e1)
				{
					((LoginWindow) sender).showMessageDialog(e1.getMessage(), e1.getClass().toString(), ERRO);
				}
				break;
			case 15:
				loginWindow = ((LoginWindow) sender);
				String passWord;
				if ((passWord = loginWindow.getRegisterPassword1()).equals(loginWindow.getRegisterPassword2()))
					try
					{
						loginInterface.loginWindowBotaoRegistar(loginWindow.getRegisterName(), passWord);
					}
					catch (IOException e1)
					{
						loginWindow.showMessageDialog(e1.getMessage(), e1.getClass().toString(), ERRO);
					}
				else
					loginWindow.showMessageDialog("Pass words doesn't match", "Invalid Auth", JOptionPane.ERROR_MESSAGE);
				break;
			case 21:
			{
				//acrescenta ao buffer das mensagens e apaga a caixa d eedio
				// e envia a mensagem
				mainInterface.mainWindowEnviaMensagem(((MainWindow) sender).sendMessage());
			}
			break;
			case 24: //boto ligar
				mainInterface.mainWindowMenuLigar();
				break;
			case 25: //boto desligar
				mainInterface.mainWindowMenuDesligar();
				break;
			case 26: //boto sair
				mainInterface.mainWindowMenuSair();
				break;
			case 27: //boto acerca
				((MainWindow) sender).mostraAcerca(mainInterface.mainWindowCredits());
				break;
			case 37:
				gameWindowInterface.gameWindowNovoJogo((GameWindow) sender);
				break;
			case 33:
				try
				{
					gameWindowInterface.gameWindowEnviaMensagem((GameWindow) sender, ((GameWindow) sender).sendMessage());
				}
				catch (IOException e1)
				{
					((GameWindow) sender).showMessageBox(e1.getMessage(), e1.getClass().toString(), ERRO);
				}
				break;
			case 36:
				if (((GameWindow) sender).perguntaSeDesiste() == JOptionPane.YES_OPTION)
					try
					{
						gameWindowInterface.gameWindowDesistir((GameWindow) sender);
					}
					catch (IOException e1)
					{
						((GameWindow) sender).showMessageBox(e1.getMessage(), e1.getClass().toString(), ERRO);
					}
				break;
			default:
				System.err.println("Erro no actionlistener");
		}
	}

	public void keyPressed(KeyEvent e)
	{
	}

	public void keyReleased(KeyEvent e)
	{
		if (actionCode == 10)
		{
			((LoginWindow) sender).updateRegisterWithLogin("servidor");
		}
		else if (actionCode == 11)
		{
			((LoginWindow) sender).updateLoginWithRegister("servidor");
		}
		else if (actionCode == 12)
		{
			((LoginWindow) sender).updateRegisterWithLogin("nome");
		}
		else if (actionCode == 13)
		{
			((LoginWindow) sender).updateLoginWithRegister("nome");
		}
	}

	public void keyTyped(KeyEvent e)
	{
		if (e.getKeyChar() == KeyEvent.VK_ENTER)
		{
			if (actionCode == 34)
			{
				try
				{
					gameWindowInterface.gameWindowEnviaMensagem((GameWindow) sender, ((GameWindow) sender).sendMessage());
				}
				catch (IOException e1)
				{
					((GameWindow) sender).showMessageBox(e1.getMessage(), e1.getClass().toString(), ERRO);
				}
			}
			else if (actionCode == 22)
			{
				mainInterface.mainWindowEnviaMensagem(((MainWindow) sender).sendMessage());
			}
		}
	}

	public void mouseDragged(MouseEvent e)
	{
	}

	public void mouseMoved(MouseEvent e)
	{
		if (actionCode == 35)
		{
			int col = calcCol(e.getX());
			if (col != -1)
				((GameWindow) sender).realcaColuna(col);
		}
	}

}
