package gui;

import javax.swing.*;
import java.awt.*;

/**
 * @author Jorge Farinha
 * @author Tiago Santos
 * @version 1.5
 * @date 24-10-2006
 */


public class GameWindow extends JFrame
{
	private final int X_OFFSET_TAB = 20;
	private final int Y_OFFSET_TAB = 15;
	private final int DIST_PECAS_TAB = 80;
	private final int TAMANHO_PECAS_TAB = 70;
	private final Color P1_COLOR = new Color(195, 0, 0);
	private final Color P2_COLOR = new Color(195, 195, 0);
	private final Color GAME_BG_COLOR = Color.BLACK;
	private final Color TAB_COLOR = new Color(0, 0, 100);
	private final Color TAB_REALCE = new Color(70, 80, 120);
	private final Color QUADRO_BG = new Color(214, 204, 204);
	private final Color QUADRO_FG = new Color(0, 0, 0);
	//private final Color BG_COLOR=new Color(200,100,50);

	private JTextPane messageBuffer;
	private JPanel contentPane;
	private JTextPane messageEditor;
	private JButton btnNewGame;
	private JButton btnDesistir;
	private int playerPoints;
	private int adversaryPoints;
	private JPanel tabuleiro;
	private JPanel quadro;
	private byte[][] positions;
	private boolean myTurn;
	private int lastColumn = -1;
	private String userName;
	private String otherUserName;
	//private int numJogadasRestantes=7*6;
	private GameWindowInterface gameWindowInterface;
	private boolean isLocked = false;
	private boolean jaMostrouMensagem = false;

	/**
	 * @param isPlayerOne   true se o jogador local  o primeiro a jogar, false se  o adversrio
	 * @param userName	  o jogador
	 * @param otherUserName o adversrio
	 * @param gameWindowInterface a implementao da interface GameWindowInterface
	 */
	public GameWindow(boolean isPlayerOne, String userName, String otherUserName, GameWindowInterface gameWindowInterface)
	{
		super("4 em linha - " + userName + " kicking @$$ of " + otherUserName);
		this.userName = userName;
		this.otherUserName = otherUserName;
		this.gameWindowInterface = gameWindowInterface;
		initializeComponent();
		positions = new byte[6][7];
		if (isPlayerOne)
			changeTurn();
		setVisible(true);
	}

	private void initializeComponent()
	{
		messageBuffer = new JTextPane();
		JScrollPane messageBufferScroll = new JScrollPane();
		JSplitPane splitPane = new JSplitPane();
		contentPane = (JPanel) this.getContentPane();
		messageEditor = new JTextPane();
		JScrollPane messageEditorScroll = new JScrollPane();
		JButton btnSend = new JButton();
		btnDesistir = new JButton();
		btnNewGame = new JButton();
		JPanel jPanel1 = new JPanel();
		tabuleiro = new JPanel();
		quadro = new JPanel();

		messageBuffer.setAutoscrolls(true);
		messageBuffer.setEditable(false);
		messageBufferScroll.setViewportView(messageBuffer);


		splitPane.setTopComponent(messageBufferScroll);
		splitPane.setBottomComponent(jPanel1);
		splitPane.setDividerLocation(180);
		splitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
		splitPane.setPreferredSize(new Dimension(400, 250));

		contentPane.setLayout(null);
//		contentPane.setBackground(BG_COLOR);

		messageEditorScroll.setViewportView(messageEditor);
		messageEditor.addKeyListener(new GameListener(34, this, gameWindowInterface));

		btnSend.setText("Enviar");
		btnSend.addActionListener(new GameListener(33, this, gameWindowInterface));

		btnDesistir.setText("Desistir");
		btnDesistir.addActionListener(new GameListener(36, this, gameWindowInterface));

		btnNewGame.setText("Novo jogo");
		btnNewGame.addActionListener(new GameListener(37, this, gameWindowInterface));
		btnNewGame.setEnabled(false);

		jPanel1.setLayout(new BoxLayout(jPanel1, BoxLayout.X_AXIS));
		jPanel1.add(messageEditorScroll, 0);
		jPanel1.add(btnSend, 1);
		//jPanel1.setBackground(BG_COLOR);

		tabuleiro.setLayout(null);
		tabuleiro.setBackground(GAME_BG_COLOR);
		tabuleiro.addMouseListener(new GameListener(32, this, gameWindowInterface));
		tabuleiro.addMouseMotionListener(new GameListener(35, this, gameWindowInterface));

		//quadro.setBackground(QUADRO_BG);
		quadro.setLayout(null);

		addComponent(contentPane, quadro, 17, 20, 260, 200);
		addComponent(contentPane, splitPane, 1, 250, 400, 250);
		addComponent(contentPane, tabuleiro, 410, 0, 590, 500);
		addComponent(contentPane, btnDesistir, 300, 20, 100, 28);
		addComponent(contentPane, btnNewGame, 300, 50, 100, 28);

		this.setLocation(new Point(0, 0));
		this.setSize(new Dimension(1010, 527));
		this.setResizable(false);

		//para fechar a janela quando se carrega na cruz
		this.addWindowListener(new GameListener(31, this, gameWindowInterface));
	}

	/**
	 * Add Component Without a Layout Manager (Absolute Positioning)
	 *
	 * @param container container
	 * @param c		 c
	 * @param x		 x
	 * @param y		 y
	 * @param width	 width
	 * @param height	heigth
	 */
	private void addComponent(Container container, Component c, int x, int y,
							  int width, int height)
	{
		c.setBounds(x, y, width, height);
		container.add(c);
	}

	/**
	 * adiciona texto  caixa com o buffer das mensagens
	 *
	 * @param text o texto a adicionar
	 */
	public void addTextToBuffer(String text)
	{
		messageBuffer.setText(messageBuffer.getText() + text + "\n");
		messageBuffer.setCaretPosition(messageBuffer.getText().length());
	}

	/**
	 * Apaga a mensagem do editor e escreve no buffer.  executado quando
	 * se carrega no boto enviar. Para enviar a mensagem para o adversrio do
	 * jogo deve ser implementado um mtodo apropriado.
	 * Se o texto do editor for \1,\2,..,\7, faz uma jogada na coluna
	 * indicada e no escreve nada no buffer.
	 *
	 * @return resposta
	 */
	public String sendMessage()
	{
		String toSend = null;
		if ((messageEditor.getText().length() != 0)
			&& !(messageEditor.getText().length() == 2 && messageEditor
				.getText().endsWith("\n")))
		{

			toSend = userName + "> " + messageEditor.getText();
			if (toSend.endsWith("\n"))
				toSend = toSend.substring(0, toSend.length() - 2);
			addTextToBuffer(toSend);

		}
		messageEditor.setText("");
		messageEditor.grabFocus();
		return toSend;
	}

	/**
	 * desenha os componentes do jogo
	 */
	public void paint(Graphics g)
	{
		super.paint(g);
		desenhaTabuleiro();
		desenhaQuadro();
	}

	/**
	 * desenha o quadro de pontuaes
	 */
	private void desenhaQuadro()
	{
		Graphics g = quadro.getGraphics();
		String fontName = g.getFont().getName();
		String nomeDoProximo;
		//desenha o quadrado
		g.setColor(QUADRO_BG);
		g.fillRoundRect(0, 0, quadro.getWidth() - 1, quadro.getHeight() - 1, 20, 20);
		g.setColor(QUADRO_FG);
		g.drawRoundRect(0, 0, quadro.getWidth() - 1, quadro.getHeight() - 1, 20, 20);
		//desenha o ranking dos 2 jogadores
		g.setFont(new Font(fontName, Font.BOLD, 20));
		g.drawString("Ranking:", 10, 40);
		g.setFont(new Font(fontName, Font.BOLD, 16));
		g.drawString(userName + ": " + playerPoints, 20, 75);
		g.drawString(otherUserName + ": " + adversaryPoints, 20, 100);
		//desenha o nome do prximo jogador
		if (isMyTurn())
			nomeDoProximo = userName;
		else
			nomeDoProximo = otherUserName;

		g.setFont(new Font(fontName, Font.BOLD, 20));
		g.drawString("Prximo: " + nomeDoProximo, 10, 155);
	}

	/**
	 * deseha o tabuleiro do jogo
	 */
	private void desenhaTabuleiro()
	{
		Graphics g = tabuleiro.getGraphics();
		//desenha o tabuleiro
		g.setColor(TAB_COLOR);
		g.fillRect(10, 10, 570, 480);
		//desenha os buracos
		for (int i = 0; i < positions.length; i++)
			for (int j = 0; j < positions[0].length; j++)
			{
				if (positions[i][j] == 0)
				{
					g.setColor(GAME_BG_COLOR);
				}
				else if (positions[i][j] == 1)
				{
					g.setColor(P1_COLOR);
				}
				else if (positions[i][j] == 2)
				{
					g.setColor(P2_COLOR);
				}
				g.fillOval(X_OFFSET_TAB + DIST_PECAS_TAB * j, Y_OFFSET_TAB + DIST_PECAS_TAB * i, TAMANHO_PECAS_TAB, TAMANHO_PECAS_TAB);
			}
	}

	/**
	 * @param col a coluna em que se quer colocar a pea
	 * @return true ou false, conforme seja ou no possvel colocar uma pea na
	 *         coluna col.
	 */
	public Point colocaPeca(int col)
	{
		Graphics g = tabuleiro.getGraphics();
		int i;
		for (i = positions.length - 1; i >= 0; i--)
		{
			if (positions[i][col] == 0 && !isLocked)
			{
				if (isMyTurn())
				{
					positions[i][col] = 1;
					g.setColor(P1_COLOR);
				}
				else
				{
					positions[i][col] = 2;
					g.setColor(P2_COLOR);
				}
//				desenha o crculo
				g.fillOval(X_OFFSET_TAB + DIST_PECAS_TAB * col, Y_OFFSET_TAB + DIST_PECAS_TAB * i, TAMANHO_PECAS_TAB, TAMANHO_PECAS_TAB);
				//numJogadasRestantes--;
				changeTurn();
				if (isMyTurn())//esta linha tem de estar depois de trocar de userName (changeTunrn())
					ganhou(new Point(i, col));
				return new Point(i, col);
			}
		}
		return new Point(-1, -1);
	}

	/**
	 * verifica se o jogo terminou e muda os enabled's dos botes
	 *
	 * @return true ou false conforme o jogo esteja ou no todo preenchido.
	 */
	public boolean checkGameOver()
	{
		boolean gOver = true;
		for (int i = 0; i < positions[0].length; i++)
			if (positions[0][i] == 0)
			{
				gOver = false;
				break;
			}
		if (gOver && !jaMostrouMensagem)
		{
			jaMostrouMensagem = true;
			btnNewGame.setEnabled(true);
			btnDesistir.setEnabled(false);
			gameWindowInterface.gameWindowFimDoJogo(this, null, null);
		}

		return gOver;
	}


	/**
	 * muda a vez do jogador actual
	 */
	public void changeTurn()
	{
		myTurn = !myTurn;
		if (this.isVisible())
			desenhaQuadro();
	}

	/**
	 * @return true ou false, conforme seja ou no o jogador local o prximo a jogar
	 */
	public boolean isMyTurn()
	{
		return myTurn;
	}

	/**
	 * verifica se o jogador que fez a ultima jogada no Point p ganhou o jogo
	 * mostra uma caixa de mensagens no ecr se tiver ganho
	 *
	 * @param p ponto
	 */
	void ganhou(Point p)
	{
		if (emLinha(positions, p.x, p.y, positions[p.x][p.y]))
		{
			setLocked(true);
			String winner, looser;
			if (isMyTurn())
			{
				winner = otherUserName;
				looser = userName;
			}
			else
			{
				winner = userName;
				looser = otherUserName;
			}
			gameWindowInterface.gameWindowFimDoJogo(this, winner, looser);
			btnNewGame.setEnabled(true);
			btnDesistir.setEnabled(false);
			jaMostrouMensagem = true;
			JOptionPane.showMessageDialog(contentPane, winner + " ganhou o jogo!", "Fim do jogo", JOptionPane.INFORMATION_MESSAGE);
			//gameWindowInterface.gameWindowNovoJogo();


		}
	}

	/**
	 * reala a coluna col do tabuleiro de jogo
	 *
	 * @param col a coluna a realar
	 */
	public void realcaColuna(int col)
	{
		if (lastColumn != col)
		{
			if (lastColumn != -1)
				desenhaColuna(lastColumn, TAB_COLOR);
			desenhaColuna(col, TAB_REALCE);
			lastColumn = col;
		}
	}

	/**
	 * desenha uma coluna do tabuleiro de jogo
	 *
	 * @param col	 a coluna a desenhar
	 * @param bgColor a cor de fundo
	 */
	private void desenhaColuna(int col, Color bgColor)
	{
		Graphics g = tabuleiro.getGraphics();
		g.setColor(bgColor);
		g.fillRect(col * DIST_PECAS_TAB + X_OFFSET_TAB - 5, Y_OFFSET_TAB - 5, DIST_PECAS_TAB, 6 * DIST_PECAS_TAB);
		for (int i = 0; i < positions.length; i++)
		{
			if (positions[i][col] == 0)
			{
				g.setColor(GAME_BG_COLOR);
			}
			else if (positions[i][col] == 1)
			{
				g.setColor(P1_COLOR);
			}
			else if (positions[i][col] == 2)
			{
				g.setColor(P2_COLOR);
			}
			g.fillOval(X_OFFSET_TAB + DIST_PECAS_TAB * col, Y_OFFSET_TAB + DIST_PECAS_TAB * i, TAMANHO_PECAS_TAB, TAMANHO_PECAS_TAB);
		}
	}

	/**
	 * apaga o tabuleiro de jogo, para se poder iniciar um novo jogo
	 */
	public void resetBoard()
	{
		//reinicializa o tabuleiro
		for (byte[] position : positions)
			for (int j = 0; j < positions[0].length; j++)
				position[j] = 0;

		isLocked = false;
		jaMostrouMensagem = false;
		//numJogadasRestantes=7*6;
		btnNewGame.setEnabled(false);
		btnDesistir.setEnabled(true);
		//redesenha o tabuleiro
		desenhaTabuleiro();
		desenhaQuadro();
	}

	/**
	 * define o ranking do jogador local a mostrar na interface grfica
	 *
	 * @param r rank do jogador local
	 */
	public void setPlayerPoints(int r)
	{
		playerPoints = r;
	}

	/**
	 * define o ranking do jogador remoto a mostrar na interface grfica
	 *
	 * @param r rank do jogador remoto
	 */
	public void setAdversaryPoints(int r)
	{
		adversaryPoints = r;
	}

	/**
	 * @return o ranking do jogador local guardado na interface grfica
	 */
	public int getPlayerPoints()
	{
		return playerPoints;
	}

	/**
	 * @return o ranking do jogador remoto guardado na interface grfica
	 */
	public int getAdversaryPoints()
	{
		return adversaryPoints;
	}

	public String getOtherUserName()
	{
		return otherUserName;
	}


	/**
	 * incrementa o ranking do jogador local em r unidades
	 *
	 * @param r o valor a adicionar ao ranking
	 * @return o novo valor do ranking
	 */
	public int increaseMyRanking(int r)
	{
		playerPoints += r;
		return playerPoints;
	}

	/**
	 * decrementa o ranking do jogador remoto em r unidades
	 *
	 * @param r o valor a adicionar ao ranking
	 * @return o novo valor do ranking
	 */
	public int increaseOtherRanking(int r)
	{
		adversaryPoints += r;
		return adversaryPoints;
	}

	///////////////////////////////////////////////////////////////////////////
	private boolean emLinha(byte tab[][], int line, int col, int codJogador)
	{
		int[][] x = new int[7][6];
		for (int i = 0; i < tab.length; i++)
			for (int j = 0; j < tab[0].length; j++)
				x[j][i] = tab[i][j];
		return em_linha(x, col, line, codJogador);
	}

	private boolean em_linha(int tab[][], int x, int y, int cod_jogador)
	{
		return horizontal(tab, x, y, cod_jogador) ||
			   vertical(tab, x, y, cod_jogador) ||
			   diagonal_um(tab, x, y, cod_jogador) ||
			   diagonal_dois(tab, x, y, cod_jogador);
	}


	//Orizontal
	private boolean horizontal(int tab[][], int x, int y, int cod_jogador)
	{
		int cont = 1;
		boolean sai = false;
		int x_ori = x;
		while (!sai && (x + 1 != 7))
		{
			x++;
			//System.out.println(tab[x][y]+"  x"+(x)+"  y"+y+"Mais");
			if (tab[x][y] == cod_jogador)
				cont++;
			else sai = true;
			if (cont == 4)
				return true;
		}
		sai = false;
		x = x_ori;
		while (!sai && (x - 1 != -1))
		{
			x--;
			//System.out.println(tab[x][y]+"  x"+(x)+"  y"+y);
			if (tab[x][y] == cod_jogador)
				cont++;
			else sai = true;
			if (cont == 4)
				return true;

		}
		return false;
	}

	//Vertical
	private boolean vertical(int tab[][], int x, int y, int cod_jogador)
	{

		int cont = 1;
		boolean sai = false;
		int y_ori = y;
		while (!sai && (y + 1 != 6))
		{
			y++;
			// System.out.println(tab[x][y]+"  x"+(x)+"  y"+y+"SOMAR");
			if (tab[x][y] == cod_jogador)
				cont++;
			else sai = true;
			if (cont == 4)
				return true;
		}
		sai = false;
		y = y_ori;
		while (!sai && (y - 1 != -1))
		{
			y--;
			//System.out.println(tab[x][y]+"  x"+x+"  y"+(y));
			if (tab[x][y] == cod_jogador)
				cont++;
			else sai = true;
			if (cont == 4)
				return true;
		}
		return false;
	}

	//Diagonal_um
	private boolean diagonal_um(int tab[][], int x, int y, int cod_jogador)
	{
		int cont = 1;
		boolean sai = false;
		int x_ori = x;
		int y_ori = y;
		while (!sai && (y + 1 != 6) && (x + 1 != 7))
		{
			x++;
			y++;
			// System.out.println(tab[x][y]+"  x"+(x)+"  y"+(y)+"nnnnnn");
			if (tab[x][y] == cod_jogador)
				cont++;
			else sai = true;
			if (cont == 4)
				return true;
		}
		sai = false;
		x = x_ori;
		y = y_ori;
		while (!sai && (x - 1 != -1) && (y - 1 != -1))
		{
			x--;
			y--;
			//System.out.println("  x"+(x)+"  y"+(y));
			if (tab[x][y] == cod_jogador)
				cont++;
			else sai = true;
			if (cont == 4)
				return true;

		}
		return false;
	}

	//Diagonal_dois
	private boolean diagonal_dois(int tab[][], int x, int y, int cod_jogador)
	{
		int cont = 1;
		boolean sai = false;
		int x_ori = x;
		int y_ori = y;
		while (!sai && x + 1 != 7 && y - 1 != -1)
		{
			x++;
			y--;
			//System.out.println(tab[x][y]+"  x"+(x)+"  y"+(y)+"nnnnnn");
			if (tab[x][y] == cod_jogador)
				cont++;
			else sai = true;
			if (cont == 4)
				return true;
		}
		sai = false;
		x = x_ori;
		y = y_ori;
		while (!sai && (x - 1 != -1) && (y + 1 != 6))
		{
			x--;
			y++;
			// System.out.println("  x"+(x)+"  y"+(y));
			if (tab[x][y] == cod_jogador)
				cont++;
			else sai = true;
			if (cont == 4)
				return true;

		}
		return false;
	}

	////////////////////////////////////////
	/**
	 * mostra uma caixe para confirmar a desistencia do jogo.
	 *
	 * @return um inteiro correspondente ao retorno do JOptionPane.showConfirmDialog
	 */
	public int perguntaSeDesiste()
	{
		int r = JOptionPane.showConfirmDialog(contentPane, "Tem a certeza que quer desistir?");
		if (r == JOptionPane.YES_OPTION)
		{
			this.setLocked(true);
			btnNewGame.setEnabled(true);
			btnDesistir.setEnabled(false);
		}
		return r;
	}

	/**
	 * bloqueia ou desbloqueia o tabuleiro de jogo para novas jogadas
	 *
	 * @param l bloqueia ou desbloqueia o tabuleiro conforme seja true ou false
	 */
	public void setLocked(boolean l)
	{
		isLocked = l;
	}

	/**
	 * mostra uma caixa de mensagens personalizada
	 *
	 * @param texto  o texto a mostra na caixa
	 * @param titulo o ttulo da caixa
	 * @param tipo   tipo de mensagem
	 */
	public void showMessageBox(String texto, String titulo, int tipo)
	{
		JOptionPane.showMessageDialog(contentPane, texto, titulo, tipo);

	}

	public void report(Exception e)
	{
		showMessageBox(e.getClass().toString(), e.getMessage(), JOptionPane.ERROR_MESSAGE);
	}

	public void update(int playerPoints, int adversaryPoints)
	{
		this.playerPoints = playerPoints;
		this.adversaryPoints = adversaryPoints;
	}
}