package gui;

import java.io.IOException;

/**
 * @author Jorge Farinha
 * @author Tiago Santos
 * @version 1.2
 * @date 13-10-2006
 */
public interface GameWindowInterface
{
	/**
	 * instrues a executar quando se fecha a janela do jogo
	 *
	 * @param gui  a janela que chama o mtodo
	 * @throws java.io.IOException no comunica com o servidor
	 */
	public void gameWindowSair(GameWindow gui) throws IOException;

	/**
	 * instrues a executar quando se carrega no botao desistir
	 *
	 * @param gui  a janela que chama o mtodo
	 * @throws java.io.IOException no comunica com o servidor
	 */
	public void gameWindowDesistir(GameWindow gui) throws IOException;

	/**
	 * instrues a executar quando se carrega no boto novo jogo
	 *
	 * @param gui  a janela que chama o mtodo
	 */
	public void gameWindowNovoJogo(GameWindow gui);

	/**
	 * instrues necessrias para enviar a mensagem para o outro jogador
	 *
	 * @param gui  a janela que chama o mtodo
	 * @param s   a mensagem a enviar
	 * @throws java.io.IOException no comunica com o servidor
	 */
	public void gameWindowEnviaMensagem(GameWindow gui, String s) throws IOException;

	/**
	 * instrues necessrias para enviar para o adversrio a coluna do
	 * tabuleiro em que o jogador local jogou
	 *
	 * @param gui  a janela que chama o mtodo
	 * @param c   a coluna onde foi feita a ltima jogada
	 * @throws java.io.IOException no comunica com o servidor
	 */
	public void gameWindowJogarPeca(GameWindow gui, int c) throws IOException;

	/**
	 * mtodo executado quando um jogador acaba o jogo
	 *
	 * @param gui	 a janela que chama o mtodo
	 * @param winner o nome do vencedor do jogo, <code>null</code> se houver empate
	 * @param looser o nome do jogador derrotado, <code>null</code> se houver empate
	 */
	public void gameWindowFimDoJogo(GameWindow gui, String winner, String looser);

}
