package gui;

import game.objects.Rank;

import javax.swing.*;
import java.awt.*;
import java.util.Hashtable;
import java.util.Vector;

/**
 * @author Jorge Farinha
 * @author Tiago Santos
 * @version 1.1
 * @date 17-10-2006
 */
public class MainWindow extends JFrame
{
	private MyList<Rank> ranking;
	private MyList<String> activeUserNames;

	private JTextPane jTextPaneHistory;
	private JTextPane jTextPaneEditor;
	private JPanel contentPane;
	private String userName;
	private MainInterface mainInterface;

	/**
	 * @param userName	  o nome do jogador
	 * @param mainInterface um objecto com a implementao dos mtodos da classe MainInterface
	 */
	public MainWindow(String userName, MainInterface mainInterface)
	{
		super("4 em linha - " + userName);
		this.userName = userName;
		this.mainInterface = mainInterface;

		initializeComponent();
	}

	/**
	 * inicializa os componentes :P
	 */
	private void initializeComponent()
	{
		ranking = new MyList<Rank>(false, true);
		activeUserNames = new MyList<String>(true, true);
		jTextPaneHistory = new JTextPane();
		jTextPaneHistory.setEditable(false);
		JScrollPane messageBufferScroll = new JScrollPane();
		JSplitPane splitPane = new JSplitPane();
		contentPane = (JPanel) this.getContentPane();
		jTextPaneEditor = new JTextPane();
		jTextPaneEditor.addKeyListener(new GameListener(22, this, mainInterface));
		JScrollPane messageEditorScroll = new JScrollPane();
		JButton btnSend = new JButton();
		JPanel jPanel1 = new JPanel();
		jTextPaneHistory.setAutoscrolls(true);
		messageBufferScroll.setViewportView(jTextPaneHistory);

		JMenuBar menuBar = new JMenuBar();


		setJMenuBar(menuBar);


		JMenu fileMenu = new JMenu("Ficheiro");
		JMenu helpMenu = new JMenu("Ajuda");
		menuBar.add(fileMenu);
		menuBar.add(helpMenu);


		JMenuItem ligar = new JMenuItem("Ligar");
		JMenuItem desligar = new JMenuItem("Desligar");
		JMenuItem sair = new JMenuItem("Sair");

		ligar.addActionListener(new GameListener(24, this, mainInterface));
		desligar.addActionListener(new GameListener(25, this, mainInterface));
		sair.addActionListener(new GameListener(26, this, mainInterface));

		fileMenu.add(ligar);
		fileMenu.add(desligar);
		fileMenu.add(sair);

		JMenuItem acerca = new JMenuItem("Acerca");

		acerca.addActionListener(new GameListener(27, this, mainInterface));

		helpMenu.add(acerca);


		splitPane.setTopComponent(messageBufferScroll);
		splitPane.setBottomComponent(jPanel1);
		splitPane.setDividerLocation(180);
		splitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
		splitPane.setPreferredSize(new Dimension(400, 250));

		contentPane.setLayout(null);
		addComponent(contentPane, splitPane, 1, 210, 400, 250);

		messageEditorScroll.setViewportView(jTextPaneEditor);

		btnSend.setText("Enviar");
		btnSend.addActionListener(new GameListener(21, this, mainInterface));

		jPanel1.setLayout(new BoxLayout(jPanel1, BoxLayout.X_AXIS));
		jPanel1.add(messageEditorScroll, 0);
		jPanel1.add(btnSend, 1);

		activeUserNames.addMouseListener(new GameListener(23, this, mainInterface));

		addComponent(contentPane, new JLabel("Clientes ligados:"), 410, 0, 290, 30);
		addComponent(contentPane, new JLabel("Ranking:"), 10, 0, 100, 30);

		addComponent(contentPane, activeUserNames, 410, 30, 290, 430);
		addComponent(contentPane, ranking, 1, 30, 400, 170);


		this.setLocation(new Point(0, 0));
		this.setSize(new Dimension(710, 512));
		this.setResizable(false);

		//para fechar a janela quando se carrega na cruz
		this.addWindowListener(new GameListener(20, this, mainInterface));
		setVisible(true);
	}

	/**
	 * Add Component Without a Layout Manager (Absolute Positioning)
	 *
	 * @param container container
	 * @param c		 c
	 * @param x		 x
	 * @param y		 y
	 * @param width	 width
	 * @param height	height
	 */
	private void addComponent(Container container, Component c, int x, int y,
							  int width, int height)
	{
		c.setBounds(x, y, width, height);
		container.add(c);
	}

	/**
	 * escreve uma string no fim do buffer das mensagens
	 *
	 * @param string o texto a escrever
	 */
	public void mainChat(String string)
	{
		jTextPaneHistory.setText(jTextPaneHistory.getText() + string + "\n");
		jTextPaneHistory.setCaretPosition(jTextPaneHistory.getText().length());
	}

	/**
	 * envia a mensagem da caixa de edio de mensagens, no escreve no buffer
	 *
	 * @return a mensagem enviada
	 */
	public String sendMessage()
	{
		String toSend = null;
		if ((jTextPaneEditor.getText().length() != 0) && !(jTextPaneEditor.getText().length() == 2 && jTextPaneEditor.getText().endsWith("\n")))
		{
			toSend = userName + "> " + jTextPaneEditor.getText();
			if (toSend.endsWith("\n"))
				toSend = toSend.substring(0, toSend.length() - 2);
		}
		jTextPaneEditor.setText("");
		jTextPaneEditor.grabFocus();
		return toSend;
	}

	/**
	 * @return devolve o nome do jogador que est seleccionado na lista de jogadores ligados
	 */
	String getSelectedPlayer()
	{
		return activeUserNames.getSelectedValue();
	}


	public void paint(Graphics g)
	{
		super.paint(g);
	}

	/**
	 * limpa a lista de clientes ligados
	 */
	public void removeAllClients()
	{
		activeUserNames.clear();
	}

	/**
	 * limpa o ranking
	 */
	public void removeAllRanking()
	{
		ranking.clear();
	}

	/**
	 * adiciona um jogador  lista de jogadores ligados
	 *
	 * @param userName jogador
	 */
	public void addPlayer(String userName)
	{
		activeUserNames.add(userName);
	}

	/**
	 * Remove um jogador da lista de jogadores ligados
	 *
	 * @param userName o nome do jogador a remover da lista
	 */
	public void removePlayer(String userName)
	{
		activeUserNames.remove(userName);
	}

	/**
	 * remove um jogador do ranking
	 *
	 * @param userName userName do jogador a remover
	 */
	public void removeClientFromRanking(String userName)
	{
		ranking.remove(new Rank(userName, 0));
	}

	/**
	 * mostra uma caixa de mensagens com informaes acerca dos autores do programa
	 *
	 * @param string o nome dos autores do programa
	 */
	void mostraAcerca(String string)
	{
		String mensagem, mensagem1 = "";
		if (string != null && string.length() > 2)
		{
			mensagem1 =
					"Autores:\n" +
					string + "\n" +
					"_____________________________________\n";
		}
		mensagem =
				"Interface grfica criada por:\n" +
				"    Jorge Farinha (jfar@student.dei.uc.pt)\n" +
				"    Tiago Santos (tjsantos@student.dei.uc.pt)\n";
		JOptionPane.showMessageDialog(contentPane, mensagem1 + mensagem, "Acerca...", JOptionPane.INFORMATION_MESSAGE);
	}

	/**
	 * mostra uma caixa de mensagens personalizada
	 *
	 * @param texto  o texto a mostra na caixa
	 * @param titulo o ttulo da caixa
	 * @param tipo   tipo de mensagem
	 */
	public void showMessageBox(String texto, String titulo, int tipo)
	{
		JOptionPane.showMessageDialog(contentPane, texto, titulo, tipo);

	}

	/**
	 * @param pergunta A string a mostrar na caixa de dilogo.
	 * @param titulo   A string a mostrar no ttulo da caixa de dilogo
	 * @return um inteiro correspondente ao boto clicado, de acordo com
	 *         a classe JOptionPane:
	 *         <code><br>
	 *         YES_OPTION = 0;<br>
	 *         NO_OPTION = 1;<br>
	 *         CLOSED_OPTION = -1;<br>
	 *         </code>
	 */
	public boolean showConfirmBox(String pergunta, String titulo)
	{
		return JOptionPane.showConfirmDialog(contentPane, userName + ":\n" + pergunta, titulo, JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION;
	}

	public void setRanking(Hashtable<String,Rank> ranking)
	{
		this.ranking.set(ranking);
	}

	public void setActiveUserNames(Vector<String> activeUserNames)
	{
		activeUserNames.remove(userName);
		this.activeUserNames.set(activeUserNames);
	}

	public void report(Exception e)
	{
		showMessageBox(userName + ":\n" + e.getMessage(), e.getClass().toString(), JOptionPane.ERROR_MESSAGE);
	}
}